package com.uas.cloud.mall.payment.api;

import com.uas.cloud.commons.domain.PageInfo;
import com.uas.cloud.commons.domain.PageParams;
import com.uas.cloud.mall.payment.domain.Payment;
import com.uas.cloud.mall.payment.service.PaymentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.text.ParseException;
import java.util.Map;

@RestController
@RequestMapping("/payment")
public class PaymentController {

    @Autowired
    private PaymentService paymentService;

    /**
     *   支付参数签名加密
     *
     *   @param orderid        订单号
     *   @param ensurePrice    付款金额
     *   @param subject        商品描述
     */
    @RequestMapping(value = "/paymentSign", method = RequestMethod.GET)
    public Map<String,String>  paymentSign(String orderid, String ensurePrice, String subject){
        return paymentService.paymentSign(orderid, ensurePrice, subject);
    }

    /**
     *   验证银盛支付异步通知，是否付款成功
     *
     *   @param request        银盛请求信息
     */
    @RequestMapping(value = "/ysepayNotify", method = RequestMethod.POST)
    public String ysepayNotify(HttpServletRequest request) throws ParseException {
        return paymentService.ysepayNotify(request);
    }

    /**
     *   发起退款
     *
     *   @param orderid        订单号
     *   @param reid           退款单号
     *   @param refundMoney   退款金额
     *   @param refundReason  退款原因
     */
    @RequestMapping(value = "/refund", method = RequestMethod.POST)
    public String refund(String orderid, String reid, String refundMoney, String refundReason) throws ParseException, IOException {
        return paymentService.refund(orderid, reid, refundMoney, refundReason);
    }

    /**
     *  查询银盛余额
     *
     */
    @RequestMapping(value = "/account", method = RequestMethod.GET)
    public Map<String, String> account() throws IOException {
        return paymentService.account();
    }

    /**
     *  查询交易记录表
     *  @param params       分页信息
     *  @param keyword      关键字
     *  @param fromDate     起始日期
     *  @param toDate       结束日期
     *
     */

    @RequestMapping(value = "/queryPayment", method = RequestMethod.GET)
    public Page<Payment> queryPayment(PageParams params, String keyword, Long fromDate, Long toDate){
        PageInfo pageInfo = new PageInfo(params);
        return paymentService.queryPayment(pageInfo,keyword,fromDate,toDate);
    }

}
