
package com.uas.cloud.mall.payment.service;

import com.uas.cloud.commons.domain.PageInfo;
import com.uas.cloud.mall.payment.domain.Payment;
import org.springframework.data.domain.Page;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.ParseException;
import java.util.Map;

public interface PaymentService {

    /**
     *   支付参数签名加密
     *
     *   @param orderid        订单号
     *   @param ensurePrice    付款金额
     *   @param subject        商品描述
     */
    Map<String, String> paymentSign(String orderid,String ensurePrice, String subject);


    /**
     *   验证银盛支付异步通知，是否付款成功
     *
     *   @param request        请求银盛请求信息
     */
    String ysepayNotify(HttpServletRequest request) throws ParseException;


    /**
     *   保存交易信息
     *
     *   @param payment   交易信息
     */
    void save(Payment payment);

    /**
     *   退款请求参数加密
     *
     *   @param orderid        订单号
     *   @param reid           退款单号
     *   @param refundMoney   退款金额
     *   @param refundReason  退款原因
     */
    Map<String, String> refundSign(String orderid,String reid, String refundMoney, String refundReason);


    /**
     *   发起退款
     *
     *   @param orderid        订单号
     *   @param reid           退款单号
     *   @param refundMoney   退款金额
     *   @param refundReason  退款原因
     */
    String refund(String orderid,String reid, String refundMoney, String refundReason) throws ParseException, IOException;


    /**
     *   查询银盛余额请求参数加密
     *
     */
    Map<String,String> accountSign();


    /**
     *   查询银盛余额
     *
     */
    Map<String,String> account() throws IOException;

     /**
     * 查询交易记录表
     *
     * @param pageInfo     分页信息
     * @param keyword      关键字
     * @param fromDate     起始日期
     * @param toDate       结束日期
     */
    public Page<Payment> queryPayment(PageInfo pageInfo, String keyword, Long fromDate, Long toDate);

}

