package com.uas.cloud.mall.payment.util;

import java.io.InputStream;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.core.io.FileSystemResource;

public class ApipaySubmit {
	/**
	 * api请求的签名工具方法，把请求参数按照字符排序拼接，然后进行RAS加密
	 *
	 * @param request
	 * @param sParaTemp
	 * @return
	 * @see
	 */
	public static String sign(HttpServletRequest request, Map<String, String> sParaTemp) {
		Map<String, String> sPara = SignUtils.paraFilter(sParaTemp);
		String signResult = null;
		InputStream pfxCertFileInputStream = null;

		try {
			pfxCertFileInputStream = new FileSystemResource(DirectPayConfig.PATH_PARTER_PKCS12).getInputStream();
			signResult = SignUtils.rsaSign(sPara, sParaTemp.get("charset"), pfxCertFileInputStream);
		} catch (Exception e) {
			throw new RuntimeException("签名失败，请检查证书文件是否存在，密码是否正确");
		}

		return signResult;
	}

	/**
	 * API验证签名工具，把签名值，请求字符编码，返回结果(json body)传递过来进行验证签名
	 *
	 * @param sign
	 * @param responseBody
	 * @param charset
	 * @return
	 * @see
	 */
	public static boolean verifySign(String sign, String responseBody, String charset) {
		InputStream publicCertFileInputStream = null;
		boolean isSign;

		try {
			publicCertFileInputStream = new FileSystemResource(DirectPayConfig.PATH_YSEPAY_PUBLIC_CERT)
					.getInputStream();
			isSign = SignUtils.rsaCheckContent(publicCertFileInputStream, responseBody, sign, charset);
		} catch (Exception e) {
			throw new RuntimeException("验证签名失败，请检查银盛公钥证书文件是否存在");
		}

		return isSign;
	}

	public static Map<String, String> buildRequestPara(Map<String, String> sParaTemp) {
		Map<String, String> sPara = SignUtils.paraFilter(sParaTemp);
		InputStream pfxCertFileInputStream = null;

		try {
			pfxCertFileInputStream = new FileSystemResource(DirectPayConfig.PATH_PARTER_PKCS12).getInputStream();
			String mysign = SignUtils.rsaSign(sPara, sParaTemp.get("charset"), pfxCertFileInputStream);
			sPara.put("sign", mysign);
		} catch (Exception e) {
			throw new RuntimeException("签名失败，请检查证书文件是否存在，密码是否正确");
		}

		return sPara;
	}

	public static boolean verifySign(Map<String, String> params) {
		InputStream publicCertFileInputStream = null;
		boolean isSign;

		try {
			publicCertFileInputStream = new FileSystemResource(DirectPayConfig.PATH_YSEPAY_PUBLIC_CERT)
					.getInputStream();
			String sign = "";
			if (params.get("sign") != null) {
				sign = params.get("sign");
			}
			System.out.println("公钥的地址" + DirectPayConfig.PATH_YSEPAY_PUBLIC_CERT);
			isSign = SignUtils.rsaCheckContent(publicCertFileInputStream, params, sign, DirectPayConfig.DEFAULT_CHARSET);
		} catch (Exception e) {
			throw new RuntimeException("验证签名失败，请检查银盛公钥证书文件是否存在");
		}

		return isSign;
	}
}
